/*----- PROTECTED REGION ID(PLCsocket.cpp) ENABLED START -----*/
static const char *RcsId = "$Id: PLCsocket.cpp,v 1.8 2017-02-23 08:42:46 graziano Exp $";
//=============================================================================
//
// file :        PLCsocket.cpp
//
// description : C++ source for the PLCsocket class and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               PLCsocket are implemented in this file.
//
// project :     PLC interlock server
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
// $Author: graziano $
//
// $Revision: 1.8 $
// $Date: 2017-02-23 08:42:46 $
//
// $HeadURL:  $
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <tango.h>
#include <PLCsocketClass.h>
#include <iostream>
#include "plclgthread.h"
#include "plctimingthread.h"
#include "plcsrvthread.h"
#include <iomanip>      //used for debug dump of db

/*----- PROTECTED REGION END -----*/	//	PLCsocket.cpp

/**
 *  PLCsocket class description:
 *    
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name         |  Method name
//================================================================
//  State                |  Inherited (no method)
//  Status               |  Inherited (no method)
//  Beam_dump            |  beam_dump
//  Send_ack             |  send_ack
//  Seq_state_rst        |  seq_state_rst
//  Test_debug           |  test_debug
//  EnableBPM11_2        |  enable_bpm11_2
//  DisableInterlockSCW  |  disable_interlock_scw
//  DisableInterlockID5  |  disable_interlock_id5
//================================================================

//================================================================
//  Attributes managed are:
//================================================================
//  IThreshold      |  Tango::DevBoolean	Scalar
//  Libera3hc       |  Tango::DevBoolean	Scalar
//  BeamDumpPLCA10  |  Tango::DevBoolean	Scalar
//  BeamAbi         |  Tango::DevBoolean	Scalar
//  CallPm          |  Tango::DevBoolean	Scalar
//  AlIThreshold    |  Tango::DevBoolean	Scalar
//  AlLibera3hc     |  Tango::DevBoolean	Scalar
//  Seq_state       |  Tango::DevString	Scalar
//  EnableBPM11_2   |  Tango::DevBoolean	Scalar
//  OpenId          |  Tango::DevBoolean	Spectrum  ( max = 16)
//  DiodeDet        |  Tango::DevBoolean	Spectrum  ( max = 16)
//  LiberaSt        |  Tango::DevBoolean	Spectrum  ( max = 16)
//  LiberaNd        |  Tango::DevBoolean	Spectrum  ( max = 16)
//  IlkEnable       |  Tango::DevBoolean	Spectrum  ( max = 16)
//  AlOpenId        |  Tango::DevBoolean	Spectrum  ( max = 16)
//  AlDiodeDet      |  Tango::DevBoolean	Spectrum  ( max = 16)
//  AlLiberaSt      |  Tango::DevBoolean	Spectrum  ( max = 16)
//  AlLiberaNd      |  Tango::DevBoolean	Spectrum  ( max = 16)
//  Seq_num         |  Tango::DevDouble	Spectrum  ( max = 3)
//================================================================

namespace PLCsocket_ns
{
/*----- PROTECTED REGION ID(PLCsocket::namespace_starting) ENABLED START -----*/

//	static initializations

/*----- PROTECTED REGION END -----*/	//	PLCsocket::namespace_starting

//--------------------------------------------------------
/**
 *	Method      : PLCsocket::PLCsocket()
 *	Description : Constructors for a Tango device
 *                implementing the classPLCsocket
 */
//--------------------------------------------------------
PLCsocket::PLCsocket(Tango::DeviceClass *cl, string &s)
 : TANGO_BASE_CLASS(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(PLCsocket::constructor_1) ENABLED START -----*/
	init_device();
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::constructor_1
}
//--------------------------------------------------------
PLCsocket::PLCsocket(Tango::DeviceClass *cl, const char *s)
 : TANGO_BASE_CLASS(cl, s)
{
	/*----- PROTECTED REGION ID(PLCsocket::constructor_2) ENABLED START -----*/
	init_device();
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::constructor_2
}
//--------------------------------------------------------
PLCsocket::PLCsocket(Tango::DeviceClass *cl, const char *s, const char *d)
 : TANGO_BASE_CLASS(cl, s, d)
{
	/*----- PROTECTED REGION ID(PLCsocket::constructor_3) ENABLED START -----*/
	init_device();
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::constructor_3
}

//--------------------------------------------------------
/**
 *	Method      : PLCsocket::delete_device()
 *	Description : will be called at device destruction or at init command
 */
//--------------------------------------------------------
void PLCsocket::delete_device()
{
	DEBUG_STREAM << "PLCsocket::delete_device() " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::delete_device) ENABLED START -----*/
	
	//	Delete device allocated objects
	//	Delete device's allocated object
	int t_finish = hbeat;// time in msec to wait for the thread to complete
	abortflag = 1; //stops thread
	
	int secs = (int)t_finish/1000;
	int nanos = (t_finish*1000-secs)*1000000000;
	omni_thread::sleep(secs,nanos);// for the last thread to finish
	usleep(11000000);
	if(attr_Seq_num_read) {
		delete attr_Seq_num_read;
		attr_Seq_num_read = 0;
	}
	if(sock)
		close(sock);
	if(new_sd)
		close(new_sd);

	if(fdlog)
		close(fdlog);
	delete mutex;
  	mysql_close(&dbp);

	/*----- PROTECTED REGION END -----*/	//	PLCsocket::delete_device
	delete[] attr_EnableBPM11_2_read;
}

//--------------------------------------------------------
/**
 *	Method      : PLCsocket::init_device()
 *	Description : will be called at device initialization.
 */
//--------------------------------------------------------
void PLCsocket::init_device()
{
	DEBUG_STREAM << "PLCsocket::init_device() create device " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::init_device_before) ENABLED START -----*/
	
	//	Initialization before get_device_property() call
		
	logloop = 0;
	tmloop = 0;
	srvloop = 0;
	fdlog = 0;
	logfile2 = "/ELETTRA_log/logs/plcsocket-srv.log";
	sock = 0;
	logging_msg = 0;
	
	abortflag = 0;	
	is_beamdumpcmd = 0;
		
	attr_OpenId_read = 0;
	attr_DiodeDet_read = 0;
	attr_LiberaSt_read = 0;
	attr_LiberaNd_read = 0;
	attr_IThreshold_read = 0;
	attr_Libera3hc_read = 0;
	attr_BeamDumpPLCA10_read = 0;
	attr_IlkEnable_read = 0;
	attr_BeamAbi_read = 0;
	attr_CallPm_read = 0;
	attr_Seq_num_read = 0;
	seq[0] = 0;//seq num
	seq[1] = 0;//seq change
	seq[2] = 0;//seq sent
	s_state = NORMAL_ST;

	message.campo_ingressi_1 = 0xffff;
	message.campo_ingressi_2 = 0xffff;
	message.campo_ingressi_3 = 0xffff;
	message.campo_ingressi_4 = 0xffff;
	message.campo_ingressi_5 = 0xffff;
	message.memo_allarmi_1 = 0xffff;
	message.memo_allarmi_2 = 0xffff;
	message.memo_allarmi_3 = 0xffff;
	message.memo_allarmi_4 = 0xffff;
	message.memo_allarmi_5 = 0xffff;
	message.campo_uscite_1 = 0x0000;
	message.campo_uscite_2 = 0xffff;



	/*----- PROTECTED REGION END -----*/	//	PLCsocket::init_device_before
	

	//	Get the device properties from database
	get_device_property();
	
	attr_EnableBPM11_2_read = new Tango::DevBoolean[1];
	/*----- PROTECTED REGION ID(PLCsocket::init_device) ENABLED START -----*/
	
	//	Initialize device

        memset (old_timestamp,0,TSTAMP_SIZE);
        memset (new_timestamp,0,TSTAMP_SIZE);
        memset (last_ck_read,0,TSTAMP_SIZE);
	if(logfile.empty())
		logfile.append(logfile2);
	if(ack_policy.empty())
		ack_policy.append(AUTO_ACK);
	
        Tango::Database *db = new Tango::Database();
        Tango::DbData db_data;
	
			
	db_data.push_back(Tango::DbDatum("openId"));
        db_data.push_back(Tango::DbDatum("diodeDet"));
        db_data.push_back(Tango::DbDatum("liberaSt"));
        db_data.push_back(Tango::DbDatum("liberaNd"));
        db_data.push_back(Tango::DbDatum("IThreshold"));
        db_data.push_back(Tango::DbDatum("libera3hc"));
        db_data.push_back(Tango::DbDatum("beamDumpPLCA10"));
        db_data.push_back(Tango::DbDatum("beamAbi"));
	
        db->get_device_attribute_property(get_name(), db_data);	//read from db attribute properties
	
	memset(&dbmessages, 0, (sizeof(dbm)));
		
	static att_db all_dbm[] = {
		{"openId", &(openid_m[0][0]), &(dbmessages.mesg_openid[0][0]), 16},
		{"diodeDet", &(diodedet_m[0][0]), &(dbmessages.mesg_diodedet[0][0]), 16},
		{"liberaSt", &(liberast_m[0][0]), &(dbmessages.mesg_liberast[0][0]), 16},
		{"liberaNd", &(liberand_m[0][0]), &(dbmessages.mesg_liberand[0][0]), 16},
		{"IThreshold", &(ithreshold_m[0]), &(dbmessages.mesg_ithreshold[0]), 1},
		{"libera3hc", &(libera3hc_m[0]), &(dbmessages.mesg_libera3hc[0]), 1},
		{"beamDumpPLCA10", &(plca10_m[0]), &(dbmessages.mesg_plca10[0]), 1},
		{"beamAbi", &(beam_dump_m[0]), &(dbmessages.mesg_beamabi[0]), 1}
	};

	for(int l=0;l<16;l++){
		memset(mesg_openid[l][0],0,35);
		memset(mesg_diodedet[l][0],0,35);
		memset(mesg_liberast[l][0],0,35);
		memset(mesg_liberand[l][0],0,35);
		memset(mesg_openid[l][1],0,35);
		memset(mesg_diodedet[l][1],0,35);
		memset(mesg_liberast[l][1],0,35);
		memset(mesg_liberand[l][1],0,35);
	}
	for(int l=0;l<2;l++){
		memset(mesg_ithreshold[l],0,35);
		memset(mesg_libera3hc[l],0,35);
		memset(mesg_plca10[l],0,35);
		memset(mesg_beamabi[l],0,35);
	}
	for (int i=0; i< db_data.size(); i++){
     		long nb_prop;
     		string &att_name = db_data[i].name;
     		db_data[i] >> nb_prop;
		for(int z=0;z<8;z++){
         		if (att_name == all_dbm[z].att_name){	//select attributes to be monitored
				int found1 = 0;
				int found2 = 0;
    				for (int k=0;k < nb_prop;k++){	//get single property
					i++;
         				string &prop_name = db_data[i].name;
					string p_val;
                			db_data[i] >> p_val;
					int pos = 0;
					int cnt = 0;
					//p_val.resize(1+p_val.length());
					//p_val.append("\n");
					if(prop_name == "db_labels_al"){	//found alarm message for db
						vector<string> stvalues;
						db_data[i] >> stvalues;
						for (int h=0; h<stvalues.size();h++){
							strcpy(&((all_dbm[z].res_m)[cnt][0]),stvalues.at(h).c_str());
							
						}
// 						for(int k=0;k<p_val.length();k++){
// 							if(p_val.c_str()[k]=='\n'){
// 								strcpy(&((all_dbm[z].res_m)[cnt][0]), ((p_val.substr(pos, k-pos)).substr(0,34)).c_str());
// 								pos = k+1;
// 								cnt+=2;
// 							}
// 						}
						found1 = 1;
					}
					cnt = 1;
					if(prop_name == "db_labels_ok"){	//found ok message for db
						vector<string> stvalues;
						db_data[i] >> stvalues;
						for (int h=0; h<stvalues.size();h++){
							strcpy(&((all_dbm[z].res_m)[cnt][0]),stvalues.at(h).c_str());
							
						}
// 						for(int k=0;k<p_val.length();k++){
// 							if(p_val.c_str()[k]=='\n'){
// 								strcpy(&((all_dbm[z].res_m)[cnt][0]), ((p_val.substr(pos, k-pos)).substr(0,34)).c_str());
// 								pos = k+1;
// 								cnt+=2;
// 							}
// 						}
						found2 = 1;
					}
				}
				if(found1 == 0){	//no alarm message for db, add default message
					for(int j=0;j<2*all_dbm[z].dim;j+=2){
						strcpy(&((all_dbm[z].res_m)[j][0]), &((all_dbm[z].fixed_m)[j][0]));
					}
				}
				if(found2 == 0){	//no ok message for db, add default message
					for(int j=1;j<2*all_dbm[z].dim;j+=2){
						strcpy(&((all_dbm[z].res_m)[j][0]), &((all_dbm[z].fixed_m)[j][0]));
					}
				}
			}
		}
	}
	
	for(int j=0, i=0;j<2*NUM_ID;j++, i++){
		strcpy(mesg_openid[i][0], &((all_dbm[0].res_m)[j][0]));
		j++;
		strcpy(mesg_openid[i][1], &((all_dbm[0].res_m)[j][0]));
	}
	for(int j=0, i=0;j<2*NUM_DIODE;j++, i++){
		strcpy(mesg_diodedet[i][0], &((all_dbm[1].res_m)[j][0]));
		j++;
		strcpy(mesg_diodedet[i][1], &((all_dbm[1].res_m)[j][0]));
	}
	for(int j=0, i=0;j<2*NUM_LIBERA1;j++, i++){
		strcpy(mesg_liberast[i][0], &((all_dbm[2].res_m)[j][0]));
		j++;
		strcpy(mesg_liberast[i][1], &((all_dbm[2].res_m)[j][0]));
	}
	for(int j=0, i=0;j<2*NUM_LIBERA2;j++, i++){
		strcpy(mesg_liberand[i][0], &((all_dbm[3].res_m)[j][0]));
		j++;
		strcpy(mesg_liberand[i][1], &((all_dbm[3].res_m)[j][0]));
	}
	strcpy(mesg_ithreshold[0], &((all_dbm[4].res_m)[0][0]));
	strcpy(mesg_ithreshold[1], &((all_dbm[4].res_m)[1][0]));
	strcpy(mesg_libera3hc[0], &((all_dbm[5].res_m)[0][0]));
	strcpy(mesg_libera3hc[1], &((all_dbm[5].res_m)[1][0]));
	strcpy(mesg_plca10[0], &((all_dbm[6].res_m)[0][0]));
	strcpy(mesg_plca10[1], &((all_dbm[6].res_m)[1][0]));
	strcpy(mesg_beamabi[0], &((all_dbm[7].res_m)[0][0]));
	strcpy(mesg_beamabi[1], &((all_dbm[7].res_m)[1][0]));
	
	
	
	logging_msg = logging;
	int fail_flag = 0;
	ip_host = ipaddr;
	char buf3[BUF_LAB_SIZE];
        memset (buf3,0,sizeof(buf3));
        memset (&message,0,sizeof(intlck_libera_t));
        memset (&old_message,0,sizeof(intlck_libera_t));
	memset (&i_answer,0,sizeof(ilk_answer_t));
	
	//mutual exclusion
	mutex = new omni_mutex::omni_mutex();
	
	
	//semaphore for message list
	argument.val = 0;
   	semid = semget(1000, 1, 0666 | IPC_CREAT);
	if(semid < 0){
		DEBUG_STREAM << "PLCsocket::cannot create semaphore" << endl;
	}
	if(semctl(semid, 0, SETVAL, argument) < 0){
		DEBUG_STREAM << "PLCsocket::Cannot init semaphore value to 0" << endl;
	}
	else{
		DEBUG_STREAM << "PLCsocket::Semaphore initialized" << endl;
	}
	
	logloop = new plclgthread::plclgthread(this);
	logloop->StartUndetached();
	
	tmloop = new plctimingthread::plctimingthread(this);
	tmloop->StartUndetached();

	if(sock > 0)
		close(sock);
	srvloop = new plcsrvthread::plcsrvthread(this);
	srvloop->StartUndetached();
	DEBUG_STREAM << "PLCsocket::init_device() started thread" << endl;


	//get mysql connection
	if(!mysql_init(&dbp)){
		if(get_state()!=Tango::FAULT)
			set_state(Tango::DISABLE);
		set_status("init db log error");
		DEBUG_STREAM << "PLCsocket::init_device() init db log error" << endl;
	}
	my_bool my_auto_reconnect=1;
	if(mysql_options(&dbp,MYSQL_OPT_RECONNECT,&my_auto_reconnect) !=0){
		if(get_state()!=Tango::FAULT)
			set_state(Tango::DISABLE);
		set_status("mysql auto reconnection error");
		DEBUG_STREAM << "PLCsocket::init_device() mysql auto reconnection error" << mysql_error(&dbp) << endl;
	}
	string dbhost;
	string dbuser;
	string dbpw;
	string dbname;
	string dbport;
	int dbportint;
	vector<string>::iterator iter = db_param.begin();
	dbhost = ((string)*iter);
	iter++;
	dbuser = ((string)*iter);
	iter++;
	dbpw = ((string)*iter);
	iter++;
	dbname = ((string)*iter);
	iter++;
	dbport = ((string)*iter);
	dbportint = atoi(dbport.c_str());
	if(dbportint!=0){
		if(!mysql_real_connect(&dbp, dbhost.c_str(), dbuser.c_str(), dbpw.c_str(), dbname.c_str(), dbportint, NULL, 0)){
//		if(!mysql_real_connect(&dbp, "log.ecs.elettra.trieste.it", "ilk", "InterLock", "FastInterlock", 3306, NULL, 0)){
			if(get_state()!=Tango::FAULT)
				set_state(Tango::DISABLE);
			set_status("db log connection error");
			DEBUG_STREAM << "PLCsocket::init_device() db log connection error" << endl;
		}
		else{
			if(get_state()==Tango::DISABLE)
				set_state(Tango::ON);
			set_status("db log connected");
		}
	}
	else{
		if(get_state()!=Tango::FAULT)
			set_state(Tango::DISABLE);
		set_status("db log connection error");
		DEBUG_STREAM << "PLCsocket::init_device() db log connection error" << endl;
	}
	
	//open the logging file
	if(logging_msg == 1){	
	
		if((fdlog = open(logfile.c_str(), O_RDWR | O_APPEND , S_IRWXU | S_IRWXG | S_IRWXO)) < 0){
			DEBUG_STREAM << "PLCsocket::init_device() cannot open log file:" << strerror(errno) << endl;
			if(get_state()!=Tango::FAULT)
				set_state(Tango::DISABLE);
			set_status("logging file opening error");
   		}
		else{
			int writefile = 0;
			int ln = 0;
			string pie;
			pie.append("seq_num\t\t");
			pie.append("seq_chng\t");
			pie.append("seq_sent\t");
			pie.append("Timestamp\t\t");
			pie.append("OpenId\t");
			pie.append("DiodeD\t");
			pie.append("LiberaS\t");
			pie.append("LiberaN\t");
			pie.append("Lib_3hc\t");
			pie.append("I<20mA\t");
			pie.append("BD_A10\t");
			pie.append("IlkEnbl\t");
			pie.append("BeamAbi\t");
			pie.append("CallPm");
			for(int j=0;j<strlen(pie.c_str());j++){
				buf3[ln+j] = pie.c_str()[j];
			}
			ln+=strlen(pie.c_str());
			buf3[ln] = '\n';
			if((writefile = write (fdlog, buf3, ln+1)<0)){
				DEBUG_STREAM << "PLCsocket::init_device() write head file failure:" << strerror(errno) << endl;
   			} 
		
		}
	}
	DEBUG_STREAM << "PLCsocket::init_device() finished" << endl;
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::init_device
}

//--------------------------------------------------------
/**
 *	Method      : PLCsocket::get_device_property()
 *	Description : Read database to initialize property data members.
 */
//--------------------------------------------------------
void PLCsocket::get_device_property()
{
	/*----- PROTECTED REGION ID(PLCsocket::get_device_property_before) ENABLED START -----*/
	
	//	Initialize property data members
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::get_device_property_before


	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("Ipaddr"));
	dev_prop.push_back(Tango::DbDatum("Wport"));
	dev_prop.push_back(Tango::DbDatum("Rport"));
	dev_prop.push_back(Tango::DbDatum("Srv_host"));
	dev_prop.push_back(Tango::DbDatum("Hbeat"));
	dev_prop.push_back(Tango::DbDatum("Labels"));
	dev_prop.push_back(Tango::DbDatum("Logfile"));
	dev_prop.push_back(Tango::DbDatum("Db_param"));
	dev_prop.push_back(Tango::DbDatum("Logging"));
	dev_prop.push_back(Tango::DbDatum("Timeadjust"));
	dev_prop.push_back(Tango::DbDatum("Ack_policy"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);
	
		//	get instance on PLCsocketClass to get class property
		Tango::DbDatum	def_prop, cl_prop;
		PLCsocketClass	*ds_class =
			(static_cast<PLCsocketClass *>(get_device_class()));
		int	i = -1;

		//	Try to initialize Ipaddr from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  ipaddr;
		else {
			//	Try to initialize Ipaddr from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  ipaddr;
		}
		//	And try to extract Ipaddr value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  ipaddr;

		//	Try to initialize Wport from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  wport;
		else {
			//	Try to initialize Wport from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  wport;
		}
		//	And try to extract Wport value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  wport;

		//	Try to initialize Rport from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  rport;
		else {
			//	Try to initialize Rport from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  rport;
		}
		//	And try to extract Rport value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rport;

		//	Try to initialize Srv_host from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  srv_host;
		else {
			//	Try to initialize Srv_host from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  srv_host;
		}
		//	And try to extract Srv_host value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  srv_host;

		//	Try to initialize Hbeat from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  hbeat;
		else {
			//	Try to initialize Hbeat from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  hbeat;
		}
		//	And try to extract Hbeat value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  hbeat;

		//	Try to initialize Labels from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  labels;
		else {
			//	Try to initialize Labels from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  labels;
		}
		//	And try to extract Labels value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  labels;

		//	Try to initialize Logfile from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  logfile;
		else {
			//	Try to initialize Logfile from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  logfile;
		}
		//	And try to extract Logfile value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  logfile;

		//	Try to initialize Db_param from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  db_param;
		else {
			//	Try to initialize Db_param from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  db_param;
		}
		//	And try to extract Db_param value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  db_param;

		//	Try to initialize Logging from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  logging;
		else {
			//	Try to initialize Logging from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  logging;
		}
		//	And try to extract Logging value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  logging;

		//	Try to initialize Timeadjust from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  timeadjust;
		else {
			//	Try to initialize Timeadjust from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  timeadjust;
		}
		//	And try to extract Timeadjust value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  timeadjust;

		//	Try to initialize Ack_policy from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  ack_policy;
		else {
			//	Try to initialize Ack_policy from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  ack_policy;
		}
		//	And try to extract Ack_policy value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  ack_policy;

	}

	/*----- PROTECTED REGION ID(PLCsocket::get_device_property_after) ENABLED START -----*/
	
	//	Check device property data members init
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::get_device_property_after
}

//--------------------------------------------------------
/**
 *	Method      : PLCsocket::always_executed_hook()
 *	Description : method always executed before any command is executed
 */
//--------------------------------------------------------
void PLCsocket::always_executed_hook()
{
	DEBUG_STREAM << "PLCsocket::always_executed_hook()  " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::always_executed_hook) ENABLED START -----*/
	
	//	code always executed before all requests
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method      : PLCsocket::read_attr_hardware()
 *	Description : Hardware acquisition for attributes
 */
//--------------------------------------------------------
void PLCsocket::read_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "PLCsocket::read_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_attr_hardware) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_attr_hardware
}

//--------------------------------------------------------
/**
 *	Read attribute IThreshold related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PLCsocket::read_IThreshold(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_IThreshold(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_IThreshold) ENABLED START -----*/
	ithreshold = (message.campo_ingressi_5 & maskbits[0])==ACTIVE?false:true;
	al_ithreshold = (message.memo_allarmi_5 & maskbits[0])==ACTIVE?false:true;
	attr_IThreshold_read = &ithreshold;
	attr.set_value(attr_IThreshold_read, 1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_IThreshold
}
//--------------------------------------------------------
/**
 *	Read attribute Libera3hc related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PLCsocket::read_Libera3hc(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_Libera3hc(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_Libera3hc) ENABLED START -----*/
	libera3hc = ((message.campo_ingressi_5 & maskbits[1])>>1)==ACTIVE?false:true;
	al_libera3hc = ((message.memo_allarmi_5 & maskbits[1])>>1)==ACTIVE?false:true;
	attr_Libera3hc_read = &libera3hc;
	attr.set_value(attr_Libera3hc_read, 1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_Libera3hc
}
//--------------------------------------------------------
/**
 *	Read attribute BeamDumpPLCA10 related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PLCsocket::read_BeamDumpPLCA10(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_BeamDumpPLCA10(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_BeamDumpPLCA10) ENABLED START -----*/
	beamdumpplca10 = ((message.campo_ingressi_5 & maskbits[2])>>2)==ACTIVE?false:true;
	attr_BeamDumpPLCA10_read = &beamdumpplca10;
	attr.set_value(attr_BeamDumpPLCA10_read, 1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_BeamDumpPLCA10
}
//--------------------------------------------------------
/**
 *	Read attribute BeamAbi related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PLCsocket::read_BeamAbi(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_BeamAbi(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_BeamAbi) ENABLED START -----*/
	beamabi = (message.campo_uscite_2 & maskbits[0])==ACTIVE?false:true;
	attr_BeamAbi_read = &beamabi;
	attr.set_value(attr_BeamAbi_read, 1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_BeamAbi
}
//--------------------------------------------------------
/**
 *	Read attribute CallPm related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PLCsocket::read_CallPm(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_CallPm(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_CallPm) ENABLED START -----*/
	callpm = ((message.campo_uscite_2 & maskbits[1])>>1)==ACTIVE?false:true;
	attr_CallPm_read = &callpm;
	attr.set_value(attr_CallPm_read, 1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_CallPm
}
//--------------------------------------------------------
/**
 *	Read attribute AlIThreshold related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PLCsocket::read_AlIThreshold(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_AlIThreshold(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_AlIThreshold) ENABLED START -----*/
	al_ithreshold = (message.memo_allarmi_5 & maskbits[0])==ACTIVE?false:true;
	attr_AlIThreshold_read = &al_ithreshold;
	attr.set_value(attr_AlIThreshold_read, 1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_AlIThreshold
}
//--------------------------------------------------------
/**
 *	Read attribute AlLibera3hc related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PLCsocket::read_AlLibera3hc(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_AlLibera3hc(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_AlLibera3hc) ENABLED START -----*/
	al_libera3hc = ((message.memo_allarmi_5 & maskbits[1])>>1)==ACTIVE?false:true;
	attr_AlLibera3hc_read = &al_libera3hc;
	attr.set_value(attr_AlLibera3hc_read, 1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_AlLibera3hc
}
//--------------------------------------------------------
/**
 *	Read attribute Seq_state related method
 *	Description: 
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PLCsocket::read_Seq_state(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_Seq_state(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_Seq_state) ENABLED START -----*/
	attr_Seq_state_read = &(s_state);
	attr.set_value(attr_Seq_state_read, 1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_Seq_state
}
//--------------------------------------------------------
/**
 *	Read attribute EnableBPM11_2 related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PLCsocket::read_EnableBPM11_2(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_EnableBPM11_2(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_EnableBPM11_2) ENABLED START -----*/
	DEBUG_STREAM << "PLCsocket::read_EnableBPM11_2: message.mesg_type= "<<hex<<message.mesg_type<<" DISABLED_BPM11_2_MESG=" <<DISABLED_BPM11_2_MESG<<dec << endl;
	if(message.mesg_type & DISABLED_BPM11_2_MESG)
		*attr_EnableBPM11_2_read = false;
	else
		*attr_EnableBPM11_2_read = true;
	//	Set the attribute value
	attr.set_value(attr_EnableBPM11_2_read);
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_EnableBPM11_2
}
//--------------------------------------------------------
/**
 *	Read attribute OpenId related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
//--------------------------------------------------------
void PLCsocket::read_OpenId(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_OpenId(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_OpenId) ENABLED START -----*/
	for(int j=0;j<NUM_ID;j++){
		openid[j] = ((message.campo_ingressi_1 & maskbits[j])>>j)==ACTIVE?false:true;
		al_openid[j] = ((message.memo_allarmi_1 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	attr_OpenId_read = &openid[0];
	attr.set_value(attr_OpenId_read, NUM_ID, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_OpenId
}
//--------------------------------------------------------
/**
 *	Read attribute DiodeDet related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
//--------------------------------------------------------
void PLCsocket::read_DiodeDet(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_DiodeDet(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_DiodeDet) ENABLED START -----*/
	for(int j=0;j<NUM_DIODE;j++){
		diodedet[j] = ((message.campo_ingressi_2 & maskbits[j])>>j)==ACTIVE?false:true;
		al_diodedet[j] = ((message.memo_allarmi_2 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	attr_DiodeDet_read = &diodedet[0];
	attr.set_value(attr_DiodeDet_read, NUM_DIODE, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_DiodeDet
}
//--------------------------------------------------------
/**
 *	Read attribute LiberaSt related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
//--------------------------------------------------------
void PLCsocket::read_LiberaSt(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_LiberaSt(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_LiberaSt) ENABLED START -----*/
	for(int j=0;j<NUM_LIBERA1;j++){
		liberast[j] = ((message.campo_ingressi_3 & maskbits[j])>>j)==ACTIVE?false:true;
		al_liberast[j] = ((message.memo_allarmi_3 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	attr_LiberaSt_read = &liberast[0];
	attr.set_value(attr_LiberaSt_read, NUM_LIBERA1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_LiberaSt
}
//--------------------------------------------------------
/**
 *	Read attribute LiberaNd related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
//--------------------------------------------------------
void PLCsocket::read_LiberaNd(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_LiberaNd(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_LiberaNd) ENABLED START -----*/
	for(int j=0;j<NUM_LIBERA2;j++){
		liberand[j] = ((message.campo_ingressi_4 & maskbits[j])>>j)==ACTIVE?false:true;
		al_liberand[j] = ((message.memo_allarmi_4 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	attr_LiberaNd_read = &liberand[0];
	attr.set_value(attr_LiberaNd_read, NUM_LIBERA2, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_LiberaNd
}
//--------------------------------------------------------
/**
 *	Read attribute IlkEnable related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
//--------------------------------------------------------
void PLCsocket::read_IlkEnable(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_IlkEnable(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_IlkEnable) ENABLED START -----*/
	for(int j=0;j<NUM_INTLOCK;j++){
		IlkEnable[j] = ((message.campo_uscite_1 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	attr_IlkEnable_read = &IlkEnable[0];
	attr.set_value(attr_IlkEnable_read, NUM_INTLOCK, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_IlkEnable
}
//--------------------------------------------------------
/**
 *	Read attribute AlOpenId related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
//--------------------------------------------------------
void PLCsocket::read_AlOpenId(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_AlOpenId(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_AlOpenId) ENABLED START -----*/
	for(int j=0;j<NUM_ID;j++){
		al_openid[j] = ((message.memo_allarmi_1 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	attr_AlOpenId_read = &al_openid[0];
	attr.set_value(attr_AlOpenId_read, NUM_ID, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_AlOpenId
}
//--------------------------------------------------------
/**
 *	Read attribute AlDiodeDet related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
//--------------------------------------------------------
void PLCsocket::read_AlDiodeDet(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_AlDiodeDet(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_AlDiodeDet) ENABLED START -----*/
	for(int j=0;j<NUM_DIODE;j++){
		al_diodedet[j] = ((message.memo_allarmi_2 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	attr_AlDiodeDet_read = &al_diodedet[0];
	attr.set_value(attr_AlDiodeDet_read, NUM_DIODE, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_AlDiodeDet
}
//--------------------------------------------------------
/**
 *	Read attribute AlLiberaSt related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
//--------------------------------------------------------
void PLCsocket::read_AlLiberaSt(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_AlLiberaSt(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_AlLiberaSt) ENABLED START -----*/
	for(int j=0;j<NUM_LIBERA1;j++){
		al_liberast[j] = ((message.memo_allarmi_3 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	attr_AlLiberaSt_read = &al_liberast[0];
	attr.set_value(attr_AlLiberaSt_read, NUM_LIBERA1, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_AlLiberaSt
}
//--------------------------------------------------------
/**
 *	Read attribute AlLiberaNd related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
//--------------------------------------------------------
void PLCsocket::read_AlLiberaNd(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_AlLiberaNd(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_AlLiberaNd) ENABLED START -----*/
	for(int j=0;j<NUM_LIBERA2;j++){
		al_liberand[j] = ((message.memo_allarmi_4 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	attr_AlLiberaNd_read = &al_liberand[0];
	attr.set_value(attr_AlLiberaNd_read, NUM_LIBERA2, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_AlLiberaNd
}
//--------------------------------------------------------
/**
 *	Read attribute Seq_num related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 3
 */
//--------------------------------------------------------
void PLCsocket::read_Seq_num(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PLCsocket::read_Seq_num(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PLCsocket::read_Seq_num) ENABLED START -----*/
	seq[0] = message.seq_num;
	seq[1] = message.seq_async;
	seq[2] = message.seq_sent;
	attr_Seq_num_read = &seq[0];
	attr.set_value(attr_Seq_num_read, 3, 0, false);
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::read_Seq_num
}

//--------------------------------------------------------
/**
 *	Method      : PLCsocket::add_dynamic_attributes()
 *	Description : Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void PLCsocket::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(PLCsocket::add_dynamic_attributes) ENABLED START -----*/
	
	//	Add your own code to create and add dynamic attributes if any
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Command Beam_dump related method
 *	Description: 
 *
 *	@returns 
 */
//--------------------------------------------------------
Tango::DevString PLCsocket::beam_dump()
{
	Tango::DevString argout;
	DEBUG_STREAM << "PLCsocket::Beam_dump()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::beam_dump) ENABLED START -----*/
	
	//	Add your own code
	argout  = new char[15];
	unsigned char query[BUF_LEN];
	memset (query,0,BUF_LEN);
	
	
	try{
		i_answer.code = BEAM_DUMP_CODE;		
		i_answer.ack_1 = 0x0;
		i_answer.ack_2 = 0x0;
		i_answer.ack_3 = 0x0;
		i_answer.ack_4 = 0x0;
		i_answer.ack_5 = 0x0;
		for(int i=0;i<8;i++){
			(i_answer.tstamp_srv)[i] = 0x00;
		}
		bcopy(&i_answer, &query[0], (sizeof(ilk_answer_t)));		
		send_frame(query, (sizeof(ilk_answer_t)));
	
	}
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("beam_dump cmd failed");
		Tango::Except::re_throw_exception(e,
										  (const char *) "write error",
										   (const char *) "write beam_dump error",
											(const char *) "PLCsocket::beam_dump", Tango::ERR);					
	}
	
	is_beamdumpcmd = 1;
	
	set_state(Tango::ON);
	set_status("Connection opened");

	strcpy(argout, "beam_dump sent");
	DEBUG_STREAM << "PLCsocket::beam_dump(): finished... !" << endl;
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::beam_dump
	return argout;
}
//--------------------------------------------------------
/**
 *	Command Send_ack related method
 *	Description: 
 *
 *	@returns 
 */
//--------------------------------------------------------
Tango::DevString PLCsocket::send_ack()
{
	Tango::DevString argout;
	DEBUG_STREAM << "PLCsocket::Send_ack()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::send_ack) ENABLED START -----*/
	
	//	Add your own code
	argout  = new char[15];
	unsigned char query[BUF_LEN];
	memset (query,0,BUF_LEN);
	
	send_auto_ack();
	
	set_state(Tango::ON);
	set_status("Connection opened");

	strcpy(argout, "acknowledge sent");
	DEBUG_STREAM << "PLCsocket::send_ack(): finished... !" << endl;
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::send_ack
	return argout;
}
//--------------------------------------------------------
/**
 *	Command Seq_state_rst related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void PLCsocket::seq_state_rst()
{
	DEBUG_STREAM << "PLCsocket::Seq_state_rst()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::seq_state_rst) ENABLED START -----*/
	
	//	Add your own code
	s_state = NORMAL_ST;
	
	DEBUG_STREAM << "PLCsocket::seq_state_rst(): set state to normal" << endl;
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::seq_state_rst
}
//--------------------------------------------------------
/**
 *	Command Test_debug related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void PLCsocket::test_debug()
{
	DEBUG_STREAM << "PLCsocket::Test_debug()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::test_debug) ENABLED START -----*/
	
	//	Add your own code
	char* bad_pointer;
	delete bad_pointer;	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::test_debug
}
//--------------------------------------------------------
/**
 *	Command EnableBPM11_2 related method
 *	Description: 
 *
 *	@param argin 
 */
//--------------------------------------------------------
void PLCsocket::enable_bpm11_2(Tango::DevBoolean argin)
{
	DEBUG_STREAM << "PLCsocket::EnableBPM11_2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::enable_bpm11_2) ENABLED START -----*/
	
	//	Add your own code
	unsigned char query[BUF_LEN];
	memset (query,0,BUF_LEN);
	
	
	try{
		if(argin)
			i_answer.code = ENABLE_BPM_11_2_CODE;
		else
			i_answer.code = DISABLE_BPM_11_2_CODE;
		i_answer.ack_1 = 0x0;
		i_answer.ack_2 = 0x0;
		i_answer.ack_3 = 0x0;
		i_answer.ack_4 = 0x0;
		i_answer.ack_5 = 0x0;
		for(int i=0;i<8;i++){
			(i_answer.tstamp_srv)[i] = 0x00;
		}
		bcopy(&i_answer, &query[0], (sizeof(ilk_answer_t)));		
		send_frame(query, (sizeof(ilk_answer_t)));
	
	}
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("EnableBPM11_2 cmd failed");
		Tango::Except::re_throw_exception(e,
										  (const char *) "write error",
										   (const char *) "write EnableBPM11_2 error",
											(const char *) "PLCsocket::EnableBPM11_2", Tango::ERR);					
	}
	
	is_beamdumpcmd = 1;
	
	set_state(Tango::ON);
	set_status("Connection opened");

	DEBUG_STREAM << "PLCsocket::EnableBPM11_2(): finished... !" << endl;
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::enable_bpm11_2
}
//--------------------------------------------------------
/**
 *	Command DisableInterlockSCW related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void PLCsocket::disable_interlock_scw()
{
	DEBUG_STREAM << "PLCsocket::DisableInterlockSCW()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::disable_interlock_scw) ENABLED START -----*/
	
	//	Add your own code
	unsigned char query[BUF_LEN];
	memset (query,0,BUF_LEN);
	
	
	try{
		i_answer.code = DISABLE_INTLK_SCW_CODE;
		i_answer.ack_1 = 0x0;
		i_answer.ack_2 = 0x0;
		i_answer.ack_3 = 0x0;
		i_answer.ack_4 = 0x0;
		i_answer.ack_5 = 0x0;
		for(int i=0;i<8;i++){
			(i_answer.tstamp_srv)[i] = 0x00;
		}
		bcopy(&i_answer, &query[0], (sizeof(ilk_answer_t)));		
		send_frame(query, (sizeof(ilk_answer_t)));
	
	}
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("Disable Interlock SCW cmd failed");
		Tango::Except::re_throw_exception(e,
										  (const char *) "write error",
										   (const char *) "write disable_interlock_scw error",
											(const char *) "PLCsocket::disable_interlock_scw", Tango::ERR);					
	}
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::disable_interlock_scw
}
//--------------------------------------------------------
/**
 *	Command DisableInterlockID5 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void PLCsocket::disable_interlock_id5()
{
	DEBUG_STREAM << "PLCsocket::DisableInterlockID5()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PLCsocket::disable_interlock_id5) ENABLED START -----*/
	
	//	Add your own code
	unsigned char query[BUF_LEN];
	memset (query,0,BUF_LEN);
	
	
	try{
		i_answer.code = DISABLE_INTLK_ID5_CODE;
		i_answer.ack_1 = 0x0;
		i_answer.ack_2 = 0x0;
		i_answer.ack_3 = 0x0;
		i_answer.ack_4 = 0x0;
		i_answer.ack_5 = 0x0;
		for(int i=0;i<8;i++){
			(i_answer.tstamp_srv)[i] = 0x00;
		}
		bcopy(&i_answer, &query[0], (sizeof(ilk_answer_t)));		
		send_frame(query, (sizeof(ilk_answer_t)));
	
	}
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("Disable Interlock ID5 cmd failed");
		Tango::Except::re_throw_exception(e,
										  (const char *) "write error",
										   (const char *) "write disable_interlock_id5 error",
											(const char *) "PLCsocket::disable_interlock_id5", Tango::ERR);					
	}
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::disable_interlock_id5
}
//--------------------------------------------------------
/**
 *	Method      : PLCsocket::add_dynamic_commands()
 *	Description : Create the dynamic commands if any
 *                for specified device.
 */
//--------------------------------------------------------
void PLCsocket::add_dynamic_commands()
{
	/*----- PROTECTED REGION ID(PLCsocket::add_dynamic_commands) ENABLED START -----*/
	
	//	Add your own code to create and add dynamic commands if any
	
	/*----- PROTECTED REGION END -----*/	//	PLCsocket::add_dynamic_commands
}

/*----- PROTECTED REGION ID(PLCsocket::namespace_ending) ENABLED START -----*/

//	Additional Methods

//+------------------------------------------------------------------
/**
 *	method:	PLCsocket::push_it
 *
 *	description:	method to push user events
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
void PLCsocket::push_it()
{
	gettimeofday(&_pre2,NULL);//takes first timestamp
	DEBUG_STREAM << "PLCsocket::push_it(): entering... !" << endl;
	vector<string> filt_names;
	vector<double> filt_values;
	filt_names.push_back("campo");
	filt_values.push_back(message.campo_ingressi_1);
			
	
	memset (buffer,0,BUF_LEN);
	bcopy(&old_message, &buffer, sizeof(PLCsocket_ns::intlck_libera));		

	for(int j=0;j<NUM_ID;j++){
		openid[j] = ((message.campo_ingressi_1 & maskbits[j])>>j)==ACTIVE?false:true;
		al_openid[j] = ((message.memo_allarmi_1 & maskbits[j])>>j)==ACTIVE?false:true;
		old_openid[j] = ((old_message.campo_ingressi_1 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	for(int j=0;j<NUM_DIODE;j++){
		diodedet[j] = ((message.campo_ingressi_2 & maskbits[j])>>j)==ACTIVE?false:true;
		al_diodedet[j] = ((message.memo_allarmi_2 & maskbits[j])>>j)==ACTIVE?false:true;
		old_diodedet[j] = ((old_message.campo_ingressi_2 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	for(int j=0;j<NUM_LIBERA1;j++){
		liberast[j] = ((message.campo_ingressi_3 & maskbits[j])>>j)==ACTIVE?false:true;
		al_liberast[j] = ((message.memo_allarmi_3 & maskbits[j])>>j)==ACTIVE?false:true;
		old_liberast[j] = ((old_message.campo_ingressi_3 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	for(int j=0;j<NUM_LIBERA2;j++){
		liberand[j] = ((message.campo_ingressi_4 & maskbits[j])>>j)==ACTIVE?false:true;
		al_liberand[j] = ((message.memo_allarmi_4 & maskbits[j])>>j)==ACTIVE?false:true;
		old_liberand[j] = ((old_message.campo_ingressi_4 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	ithreshold = (message.campo_ingressi_5 & maskbits[0])==ACTIVE?false:true;
	old_ithreshold = (old_message.campo_ingressi_5 & maskbits[0])==ACTIVE?false:true;
	al_ithreshold = (message.memo_allarmi_5 & maskbits[0])==ACTIVE?false:true;
	libera3hc = ((message.campo_ingressi_5 & maskbits[1])>>1)==ACTIVE?false:true;
	beamdumpplca10 = ((message.campo_ingressi_5 & maskbits[2])>>2)==ACTIVE?false:true;
	old_libera3hc = ((old_message.campo_ingressi_5 & maskbits[1])>>1)==ACTIVE?false:true;
	old_beamdumpplca10 = ((old_message.campo_ingressi_5 & maskbits[2])>>2)==ACTIVE?false:true;
	al_libera3hc = ((message.memo_allarmi_5 & maskbits[1])>>1)==ACTIVE?false:true;
	for(int j=0;j<NUM_INTLOCK;j++){
		IlkEnable[j] = ((message.campo_uscite_1 & maskbits[j])>>j)==ACTIVE?false:true;
		old_IlkEnable[j] = ((old_message.campo_uscite_1 & maskbits[j])>>j)==ACTIVE?false:true;
	}
	beamabi = (message.campo_uscite_2 & maskbits[0])==ACTIVE?false:true;
	old_beamabi = (old_message.campo_uscite_2 & maskbits[0])==ACTIVE?false:true;
	callpm = ((message.campo_uscite_2 & maskbits[1])>>1)==ACTIVE?false:true;
	

	//push_event makes a set_value of the attribute with the specified data, then send event
	try{
		string nm = "openId";
	
		push_event(nm, filt_names, filt_values, openid, NUM_ID);
		
		nm = "diodeDet";
		push_event(nm, filt_names, filt_values, diodedet, NUM_DIODE);
	
		nm = "liberaSt";
		push_event(nm, filt_names, filt_values, liberast, NUM_LIBERA1);
	
		nm = "liberaNd";
		push_event(nm, filt_names, filt_values, liberand, NUM_LIBERA2);
	
		nm = "IlkEnable";
		push_event(nm, filt_names, filt_values, IlkEnable, NUM_INTLOCK);

		nm = "ithreshold";
		push_event(nm, filt_names, filt_values, &ithreshold, 1);
	
		nm = "libera3hc";
		push_event(nm, filt_names, filt_values, &libera3hc, 1);
	
		nm = "beamDumpPLCA10";
		push_event(nm, filt_names, filt_values, &beamdumpplca10, 1);

		nm = "beamAbi";
		push_event(nm, filt_names, filt_values, &beamabi, 1);
	
		nm = "callPm";
		push_event(nm, filt_names, filt_values, &callpm, 1);
	
		nm = "AlOpenId";
		push_event(nm, filt_names, filt_values, al_openid, NUM_ID);
	
		nm = "AlDiodeDet";
		push_event(nm, filt_names, filt_values, al_diodedet, NUM_DIODE);
	
		nm = "AlLiberaSt";
		push_event(nm, filt_names, filt_values, al_liberast, NUM_LIBERA1);
	
		nm = "AlLiberaNd";
		push_event(nm, filt_names, filt_values, al_liberand, NUM_LIBERA2);
	
		nm = "AlIThreshold";
		push_event(nm, filt_names, filt_values, &al_ithreshold, 1);
	
		nm = "AlLibera3hc";
		push_event(nm, filt_names, filt_values, &al_libera3hc, 1);
		
	}
	catch(Tango::DevFailed e) {
		DEBUG_STREAM << "PLCsocket::push_it(): error pushing user events" << endl;
		set_state(Tango::FAULT);
		set_status("Events not pushed");
		Tango::Except::re_throw_exception(e,
										  (const char *) "pushing events error",
										   (const char *) "pushing events error",
											(const char *) "PLCsocket::push_it()", Tango::ERR);					
	}
	// corrected libera signal
	for(int j=0;j<NUM_LIBERA1;j++){
		corr_liberast[j] = (liberast[j] || IlkEnable[j]);
		old_corr_liberast[j] = (old_liberast[j] || old_IlkEnable[j]);
	}
	for(int j=0;j<NUM_LIBERA2;j++){
		corr_liberand[j] = (liberand[j] || IlkEnable[j]);
		old_corr_liberand[j] = (old_liberand[j] || old_IlkEnable[j]);
	}
	corr_libera3hc = (libera3hc || ithreshold);
	old_corr_libera3hc = (old_libera3hc || old_ithreshold);
	
	filt_names.clear();
	filt_values.clear();
	if(get_state()!=Tango::FAULT)
		set_state(Tango::ON);
	set_status("Events pushed");
	INFO_STREAM << "PLCsocket::push_it(): Events pushed" << endl;
	gettimeofday(&_post2,NULL);//takes second timestamp
	
	int tempo = (_post2.tv_sec - _pre2.tv_sec)*1000000 + 
			(int)((float)(_post2.tv_usec -	_pre2.tv_usec)) ;
	gettimeofday(&_pre2,NULL);//takes first timestamp
	
	if(message.mesg_type & CHANGE_MESG){
	
		int condition[9];
		condition[0] = message.campo_ingressi_1^old_message.campo_ingressi_1;
		condition[1] = message.campo_ingressi_2^old_message.campo_ingressi_2;
		condition[2] = message.campo_ingressi_3^old_message.campo_ingressi_3;
		condition[3] = message.campo_ingressi_4^old_message.campo_ingressi_4;
		condition[4] = message.campo_ingressi_5^old_message.campo_ingressi_5;
		condition[5]=((message.campo_uscite_2&maskbits[0])^(old_message.campo_uscite_2&maskbits[0]));
		condition[6] = message.campo_uscite_1;
		
		
		if(condition[0]!=0x0000){
			for(int j=0;j<NUM_ID;j++){
				if(openid[j]!=old_openid[j]){
					log_to_db(openid[j]<old_openid[j]?mesg_openid[j][0]:mesg_openid[j][1], message.tstamp_campo);
				}
			}
		}
		if(condition[1]!=0x0000){
			for(int j=0;j<NUM_DIODE;j++){
				if(diodedet[j]!=old_diodedet[j]){
					log_to_db(diodedet[j]<old_diodedet[j]?mesg_diodedet[j][0]:mesg_diodedet[j][1], message.tstamp_campo);
				}
			}
		}
		if((condition[2]!=0x0000) || (condition[6]!=0x0000)) {
			for(int j=0;j<NUM_LIBERA1;j++){
				if((corr_liberast[j]!=old_corr_liberast[j])){
					log_to_db(corr_liberast[j]<old_corr_liberast[j]?mesg_liberast[j][0]:mesg_liberast[j][1], message.tstamp_campo);
				}
			}
		}
		if((condition[3]!=0x0000)||(condition[6]!=0x0000)){
			for(int j=0;j<NUM_LIBERA2;j++){
				if(corr_liberand[j]!=old_corr_liberand[j]){
					log_to_db(corr_liberand[j]<old_corr_liberand[j]?mesg_liberand[j][0]:mesg_liberand[j][1], message.tstamp_campo);
				}
			}
		}
		if(condition[4]!=0x0000){
			if(ithreshold!=old_ithreshold){
				log_to_db(ithreshold<old_ithreshold?mesg_ithreshold[0]:mesg_ithreshold[1], message.tstamp_campo);
			}
			if(corr_libera3hc!=old_corr_libera3hc){
				log_to_db(corr_libera3hc<old_corr_libera3hc?mesg_libera3hc[0]:mesg_libera3hc[1], message.tstamp_campo);
			}
			if(beamdumpplca10!=old_beamdumpplca10){
				log_to_db(beamdumpplca10<old_beamdumpplca10?mesg_plca10[0]:mesg_plca10[1], message.tstamp_campo);
			}
		}
		if(condition[5]!=0x0000){
			if(beamabi!=old_beamabi){
				log_to_db(beamabi<old_beamabi?mesg_beamabi[0]:mesg_beamabi[1], message.tstamp_campo);
			}
		}
		
	
		DEBUG_STREAM << "PLCsocket::push_it() change message logged" << endl;
	
	}
	if(message.mesg_type & BDCA_MESG){
		if(beamabi<old_beamabi){
			if(is_beamdumpcmd!=0){
				log_to_db(beam_dump_cmd, message.tstamp_campo);
				is_beamdumpcmd=0;
				DEBUG_STREAM << "PLCsocket::push_it() beam dump cmd message logged" << endl;
			}
			else{
				DEBUG_STREAM << "PLCsocket::push_it() error: beam_dump_cmd_answer without the command" << endl;
			}
		}
		else{
			DEBUG_STREAM << "PLCsocket::push_it() beam dump cmd answer without beam dump" << endl;
		}
	}
	if(message.mesg_type & PERIODIC_MESG){
               //dont log
		DEBUG_STREAM << "PLCsocket::push_it() periodic message" << endl;
	}
	//if(logging_msg == 1){
//		log_it();
	//}
	DEBUG_STREAM << "PLCsocket::push_it(): finished" << endl;
	gettimeofday(&_post2,NULL);//takes second timestamp
	tempo = (_post2.tv_sec - _pre2.tv_sec)*1000000 + 
			(int)((float)(_post2.tv_usec -	_pre2.tv_usec)) ;
	
}

//+------------------------------------------------------------------
/**
 *	method:	PLCsocket::log_it
 *
 *	description:	method to execute the logging to a textfile
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
void PLCsocket::log_it()
{
	DEBUG_STREAM << "PLCsocket::log_it(): entering... !" << endl;
	char buf[10];//buffer for single digit conversion
	char buf2[BUF_LEN];
	
	int col[10];
	col[0] = message.campo_ingressi_1;//OpenID
	col[1] = message.campo_ingressi_2;//DiodeDet
	col[2] = message.campo_ingressi_3;//LiberaSt
	col[3] = message.campo_ingressi_4;//LiberaNd
	col[4] = ((message.campo_ingressi_5 & maskbits[1])>>1);//Libera3hc
	col[5] = (message.campo_ingressi_5 & maskbits[0]);//IThreshold
	col[6] = ((message.campo_ingressi_5 & maskbits[2])>>2);//BeamDumpPLCA10
	col[7] = message.campo_uscite_1;//IlkEnable
	col[8] = (message.campo_uscite_2 & maskbits[0]);//BeamAbi
	col[9] = ((message.campo_uscite_2 & maskbits[1])>>1);//CallPm
	
	int writefile = 0;
	string x;
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%08ld\t", message.seq_num);
	x.append(buf);
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%08ld\t", message.seq_async);
	x.append(buf);
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%08ld\t", message.seq_sent);
	x.append(buf);
	memset (buf2, 0, BUF_LEN );
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%02x/%02x/%02x ", message.tstamp_campo[3], message.tstamp_campo[0], message.tstamp_campo[1]);
	x.append(buf);
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%02x:%02x:%02x", message.tstamp_campo[2], message.tstamp_campo[5], message.tstamp_campo[4]);
	x.append(buf);
	memset (buf, 0, sizeof(buf) );
//		sprintf(buf, ".%02x%02x", message.tstamp_campo[7], message.tstamp_campo[6]);
	sprintf(buf, ".%02x%01x-%01x", message.tstamp_campo[7], message.tstamp_campo[6]>>4, message.tstamp_campo[6]&0x000f);
	x.append(buf);
/*	for(int k=0; k<TSTAMP_SIZE/2;k++){
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%02x-%02x-", message.tstamp_campo[1+2*k], message.tstamp_campo[2*k]);
	x.append(buf);
}*/
	sprintf(buf2, "\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d",col[0],col[1],col[2],col[3],col[4],col[5],col[6],col[7],col[8],col[9]);
	x.append(buf2);
	x.append("\n");
	
	if((writefile = write (fdlog, x.c_str(), x.length()))<0){
		if(get_state()!=Tango::FAULT)
			set_state(Tango::DISABLE);
		set_status("File logging failure");
		DEBUG_STREAM << "PLCsocket::log_it(): File logging failure" << endl;
	}
	else{
		if(get_state()!=Tango::FAULT)
			set_state(Tango::ON);
		set_status("Log written");
	}
	DEBUG_STREAM << "PLCsocket::log_it(): finished" << endl;
	
}

//+------------------------------------------------------------------
/**
 *	method:	PLCsocket::log_msg
 *
 *	description:	method to execute the logging to a textfile
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
void PLCsocket::log_msg(intlck_libera_t me)
{
	DEBUG_STREAM << "PLCsocket::log_msg(): entering... !" << endl;
	char buf[10];//buffer for single digit conversion
	char buf2[BUF_LEN];
	
	int col[10];
	col[0] = me.campo_ingressi_1;//OpenID
	col[1] = me.campo_ingressi_2;//DiodeDet
	col[2] = me.campo_ingressi_3;//LiberaSt
	col[3] = me.campo_ingressi_4;//LiberaNd
	col[4] = ((me.campo_ingressi_5 & maskbits[1])>>1);//Libera3hc
	col[5] = (me.campo_ingressi_5 & maskbits[0]);//IThreshold
	col[6] = ((me.campo_ingressi_5 & maskbits[2])>>2);//BeamDumpPLCA10
	col[7] = me.campo_uscite_1;//IlkEnable
	col[8] = (me.campo_uscite_2 & maskbits[0]);//BeamAbi
	col[9] = ((me.campo_uscite_2 & maskbits[1])>>1);//CallPm
	
	int writefile = 0;
	string x;
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%08ld\t", me.seq_num);
	x.append(buf);
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%08ld\t", me.seq_async);
	x.append(buf);
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%08ld\t", me.seq_sent);
	x.append(buf);
	memset (buf2, 0, BUF_LEN );
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%02x/%02x/%02x ", me.tstamp_campo[3], me.tstamp_campo[0], me.tstamp_campo[1]);
	x.append(buf);
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%02x:%02x:%02x", me.tstamp_campo[2], me.tstamp_campo[5], me.tstamp_campo[4]);
	x.append(buf);
	memset (buf, 0, sizeof(buf) );
//		sprintf(buf, ".%02x%02x", me.tstamp_campo[7], me.tstamp_campo[6]);
	sprintf(buf, ".%02x%01x-%01x", me.tstamp_campo[7], me.tstamp_campo[6]>>4, me.tstamp_campo[6]&0x000f);
	x.append(buf);
/*	for(int k=0; k<TSTAMP_SIZE/2;k++){
	memset (buf, 0, sizeof(buf) );
	sprintf(buf, "%02x-%02x-", me.tstamp_campo[1+2*k], me.tstamp_campo[2*k]);
	x.append(buf);
}*/
	sprintf(buf2, "\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d",col[0],col[1],col[2],col[3],col[4],col[5],col[6],col[7],col[8],col[9]);
	x.append(buf2);
	x.append("\n");
	
	if((writefile = write (fdlog, x.c_str(), x.length()))<0){
		if(get_state()!=Tango::FAULT)
			set_state(Tango::DISABLE);
		set_status("File logging failure");
		DEBUG_STREAM << "PLCsocket::log_msg(): File logging failure" << endl;
	}
	if(writefile==-1){
		set_state(Tango::DISABLE);
		set_status("File logging failure");
	}
	else if(get_state()!=Tango::FAULT){
		set_state(Tango::ON);
		set_status("Log written");
	}
	DEBUG_STREAM << "PLCsocket::log_msg(): finished" << endl;
	
}

//+------------------------------------------------------------------
/**
 *	method:	PLCsocket::log_to_db
 *
 *	description:	method to execute the logging to a textfile
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
void PLCsocket::log_to_db(db_mesg ms, tstamp_t t)
{
	DEBUG_STREAM << "PLCsocket::log_to_db(): entering... !" << endl;
	
	char query[DB_QUERY_LEN];
	int col[10];
	col[0] = message.campo_ingressi_1;//OpenID
	col[1] = message.campo_ingressi_2;//DiodeDet
	col[2] = message.campo_ingressi_3;//LiberaSt
	col[3] = message.campo_ingressi_4;//LiberaNd
	col[4] = ((message.campo_ingressi_5 & maskbits[1])>>1);//Libera3hc
	col[5] = (message.campo_ingressi_5 & maskbits[0]);//IThreshold
	col[6] = ((message.campo_ingressi_5 & maskbits[2])>>2);//BeamDumpPLCA10
	col[7] = message.campo_uscite_1;//IlkEnable
	col[8] = (message.campo_uscite_2 & maskbits[0]);//BeamAbi
	col[9] = ((message.campo_uscite_2 & maskbits[1])>>1);//CallPm
 	
	
	
	long lll = calc_timeval(t);
	long mmm = calc_time_ms(t);
	double dob = ((double)mmm)/1000;
	dob += lll;
	
	char log_m[35];
	for(int s=0;s<35;s++){
		log_m[s] = ms[s];
	}
		
	int y = snprintf(query, DB_QUERY_LEN, "INSERT INTO FastInterlock.Log (Timestamp,Message,OpenID,DiodeDet,LiberaSt,LiberaNd,Libera3hc,IThreshold,BDplcA10,IlkEnable,BeamAbi,CallPm)\
			VALUES (%f,\"%s\",%d,%d,%d,%d,%d,%d,%d,%d,%d,%d)",dob,log_m,col[0],col[1],col[2],col[3],col[4],col[5],col[6],col[7],col[8],col[9]);
	if(y>=DB_QUERY_LEN){
		DEBUG_STREAM << "PLCsocket::log_to_db(): db query too long" << endl;
	}
	else{
		if ((mysql_query(&dbp,query))){
			if(get_state()!=Tango::FAULT)
				set_state(Tango::DISABLE);
			set_status("db log insert error");
			DEBUG_STREAM << "PLCsocket::log_to_db(): " << mysql_error(&dbp) << endl;
		}
		else{
			if(get_state()!=Tango::FAULT)
				set_state(Tango::ON);
			set_status("db log insert ok");
		}
		
	}
	INFO_STREAM << "PLCsocket::log_to_db(): Events logged" << endl;
	
}


//+------------------------------------------------------------------
/**
 *	method:	PLCsocket::sync_plc
 *
 *	description:	method to refresh plc timestamp 
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
void PLCsocket::sync_plc(tstamp_t t)
{
	DEBUG_STREAM << "PLCsocket::sync_plc(): entering... !" << endl;
	
	unsigned char query[BUF_LEN];
	memset (query,0,BUF_LEN);

	try{
		i_answer.code = SYNC_CODE;		
		i_answer.ack_1 = 0x0;
		i_answer.ack_2 = 0x0;
		i_answer.ack_3 = 0x0;
		i_answer.ack_4 = 0x0;
		i_answer.ack_5 = 0x0;
		for(int i=0;i<8;i++){
			(i_answer.tstamp_srv)[i] = t[i];
		}
		
		bcopy(&i_answer, &query[0], (sizeof(ilk_answer_t)));		
		send_frame(query, (sizeof(ilk_answer_t)));
	
	}
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("Write failed");
		Tango::Except::re_throw_exception(e,
										  (const char *) "write error",
										   (const char *) "write sync_plc error",
											(const char *) "PLCsocket::sync_plc", Tango::ERR);					
	}
	
	set_state(Tango::ON);
	set_status("plc clock syncronized");


	INFO_STREAM << "PLCsocket::sync_plc(): Events logged" << endl;
	
}


//+------------------------------------------------------------------
/**
 *	method:	PLCsocket::send_frame()
 *
 * Description:	Send a  frame to a node via  TCP/IP
 *
 * Arg(s) In:	unsigned char *query -  data to send
 *		short query_length - number of data words to send
 *
 *
 */
//+------------------------------------------------------------------

void PLCsocket::send_frame (unsigned char *query, short query_length)
{
	unsigned char frame[BUF_LEN];
	memset ( frame, 0, sizeof(frame) );
	unsigned short int sbuf[BUF_LEN];
	memset (sbuf,0,1+(2*query_length));
	bcopy(&query[0], &sbuf, 2*query_length);
	for (int i=0; i<query_length; i++){
		sbuf[i] = ntohs(sbuf[i]);
	}
	bcopy(&sbuf, &frame[0], 2*query_length);

	char *buf=(char *)&sbuf;
	for(int k=0;k<query_length;)
	{
		stringstream tmp;
		int g;
		tmp << "db52: "<<setw(3)<<setfill(' ')<<k<<" -> ";
		for(g=0; g<10; g++)
		{
			if(k+g>=query_length)
				break;
			tmp << " " <<hex<<setw(2)<<setfill(' ')<< (int)(unsigned char)buf[k+g];
		}
		k+=g;
		if(k<query_length)
		{
			tmp << " |  "<<dec<<setw(3)<<setfill(' ')<<k<<hex<<" -> ";
			for(g=0; g<10; g++)
			{
				if(k+g>=query_length)
					break;
				tmp << " " <<hex<<setw(2)<<setfill(' ')<< (int)(unsigned char)buf[k+g];
			}
			k+=g;
		}
		DEBUG_STREAM<<tmp.str()<<dec<<endl;
	}
	
	int writecnt = write (new_sd, frame,query_length);
	if(writecnt ==-1){
		if(get_state()!=Tango::FAULT)
			set_state(Tango::UNKNOWN);
		set_status("error writing socket");
	}
	
}

//+------------------------------------------------------------------
/**
 *	method:	PLCsocket::send_auto_ack
 *
 *	description:	method to execute "Send_ack"
 *
 * @param	argin	hex string representing single acknowledges
 * @return	command result
 *
 */
//+------------------------------------------------------------------
void PLCsocket::send_auto_ack(void)
{
	DEBUG_STREAM << "PLCsocket::send_auto_ack(): entering... !" << endl;

	//	Add your own code to control device here

	unsigned char query[BUF_LEN];
	memset (query,0,BUF_LEN);

	try{
		i_answer.code = ACK_CODE;		
		for(int i=0;i<8;i++){
			(i_answer.tstamp_srv)[i] = 0x00;
		}
		bcopy(&i_answer, &query[0], (sizeof(ilk_answer_t)));		
		send_frame(query, (sizeof(ilk_answer_t)));
	
	}
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("Write failed");
		Tango::Except::re_throw_exception(e,
										  (const char *) "write error",
										   (const char *) "write ack error",
											(const char *) "PLCsocket::send_ack", Tango::ERR);					
	}
	
	set_state(Tango::ON);
	set_status("acknowledge sent");

	return;
}



long int PLCsocket::calc_timeval(PLCsocket_ns::tstamp_t t_new)
{
	char buffer [10];
	tm t2;
	time_t tn;

	
	memset (buffer,0,sizeof(buffer));
	sprintf (buffer, "%x", t_new[4]);
	t2.tm_sec = atoi(buffer);
	memset (buffer,0,sizeof(buffer));
	sprintf (buffer, "%x", t_new[5]);
	t2.tm_min = atoi(buffer);
	memset (buffer,0,sizeof(buffer));
	sprintf (buffer, "%x", t_new[2]);
	t2.tm_hour = atoi(buffer);
	memset (buffer,0,sizeof(buffer));
	sprintf (buffer, "%x", t_new[3]);
	t2.tm_mday = atoi(buffer);
	memset (buffer,0,sizeof(buffer));
	sprintf (buffer, "%x", t_new[0]);
	t2.tm_mon = atoi(buffer)-1;
	memset (buffer,0,sizeof(buffer));
	sprintf (buffer, "%x", t_new[1]);
	t2.tm_year = atoi(buffer)+100;
	memset (buffer,0,sizeof(buffer));
	sprintf (buffer, "%x", t_new[6]);
	t2.tm_wday = atoi(buffer)-1;
#if 0
	t2.tm_isdst = 0/*-1*/;	//TODO: -1: let mktime decide if DST and adjust, 0: leave as it is
	
	DEBUG_STREAM << __func__<<": hour=0x"<<hex<<(int)hour<<" min=0x"<<(int)min<<" sec=0x"<<(int)sec<<dec<<endl;
#endif
	tn = mktime(&t2)/*-timezone*/;//with -timezone as if GMT
#if 0
	DEBUG_STREAM << __func__<<": --> tn="<<tn<<" timezone="<<timezone<<" t2.tm_isdst="<<(int)t2.tm_isdst<<endl;
	if(t2.tm_isdst == 1)
	{
		DEBUG_STREAM << __func__<<": --> ORA SOLARE! corrected to hour="<<(int)t2.tm_hour<<" min="<<(int)t2.tm_min<<" sec="<<(int)t2.tm_sec<<endl;
	}
#endif
	return  (long int)tn;
}	/* end calc_timeval() */


long int PLCsocket::calc_time_ms(PLCsocket_ns::tstamp_t t_new)
{
	long d = 0;
	long ms_new;
	int c1, c2;
	ms_new = 0;
	memset (buffer,0,sizeof(buffer));
	sprintf (buffer, "%x", t_new[7]);
	c1 = atoi(buffer);
	memset (buffer,0,sizeof(buffer));
	sprintf (buffer, "%x", t_new[6]);
	c2 = atoi(buffer);
	ms_new = c1*10+(c2>>4);
	d = ms_new;
	
	return d;

}	/* end calc_time_ms() */

/*----- PROTECTED REGION END -----*/	//	PLCsocket::namespace_ending
} //	namespace
