/*----- PROTECTED REGION ID(PLCsocket.h) ENABLED START -----*/
//=============================================================================
//
// file :        PLCsocket.h
//
// description : Include file for the PLCsocket class
//
// project :     PLC interlock server
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
// $Author: graziano $
//
// $Revision: 1.7 $
// $Date: 2017-02-23 08:42:46 $
//
// $HeadURL:  $
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef PLCsocket_H
#define PLCsocket_H

#include <tango.h>
#include <sys/time.h>
#include <time.h>
#include <sys/sem.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <deque>
#include <mysql.h>



//size of the message buffer
#define	BUF_LEN	250
//timestamp size in bytes
#define	TSTAMP_SIZE	8
//size of the label buffer
#define	BUF_LAB_SIZE	1024
//max size of the db query
#define	DB_QUERY_LEN	1024

//beam dump and ack codes for plc
#define BEAM_DUMP_MESG		"UU"
#define ACK_MESG		"aa"
#define SYNC_MESG		"ww"
//beam dump and ack codes for plc
#define BEAM_DUMP_CODE		0x5555
#define ACK_CODE		0x6161
#define SYNC_CODE		0x7777
#define ENABLE_BPM_11_2_CODE	0x5544
#define DISABLE_BPM_11_2_CODE	0x4455
#define DISABLE_INTLK_SCW_CODE	0x1111	//4369
#define DISABLE_INTLK_ID5_CODE	0x2222	//8738
//ack policy, auto_ack by default
#define AUTO_ACK		"auto"
#define MANUAL_ACK		"manual"
//seq number state
#define NORMAL_ST		"normal state"
#define LOSTPK_ST		"lost packet"
//length of the ack message
#define ACK_MASK_LEN	10
//types of message from plc
#define PERIODIC_MESG 	0x01
#define CHANGE_MESG 	0x02
#define BDCA_MESG 	0x04 //answer message to beam dump command
#define DISABLED_BPM11_2_MESG 	0x8 //answer message to beam dump command
//number of bit of input
#define NUM_ID		16
#define NUM_DIODE	16
#define NUM_LIBERA1	16
#define NUM_LIBERA2	16
#define NUM_INTLOCK	16

//mask for operation on single bits
#define MASK_BIT0     0x0001
#define MASK_BIT1     0x0002
#define MASK_BIT2     0x0004
#define MASK_BIT3     0x0008
#define MASK_BIT4     0x0010
#define MASK_BIT5     0x0020
#define MASK_BIT6     0x0040
#define MASK_BIT7     0x0080
#define MASK_BIT8     0x0100
#define MASK_BIT9     0x0200
#define MASK_BIT10     0x0400
#define MASK_BIT11     0x0800
#define MASK_BIT12     0x1000
#define MASK_BIT13     0x2000
#define MASK_BIT14     0x4000
#define MASK_BIT15     0x8000

#define ACTIVE		0 //ACTIVE level causes boolean attributes set to false
#define UNACTIVE	1 //ACTIVE level causes boolean attributes set to true

class PLCsocketClass;
class plclgthread;
class plctimingthread;
class plcsrvthread;


/*----- PROTECTED REGION END -----*/	//	PLCsocket.h

/**
 *  PLCsocket class description:
 *    
 */

namespace PLCsocket_ns
{
/*----- PROTECTED REGION ID(PLCsocket::Additional Class Declarations) ENABLED START -----*/

//	Additional Class Declarations
// timestamp
typedef unsigned char tstamp_t[TSTAMP_SIZE];

typedef char db_mesg[35];

typedef struct intlck_libera {
	u_short campo_ingressi_1,
		campo_ingressi_2,
		campo_ingressi_3,
		campo_ingressi_4,
		campo_ingressi_5; //valori degli ingressi, bit per bit
	u_short campo_uscite_1,
		campo_uscite_2; //valori delle uscite, bit per bit
	tstamp_t tstamp_campo; //timestamp lettura ingressi
	u_short memo_allarmi_1,
		memo_allarmi_2,
		memo_allarmi_3,
		memo_allarmi_4,
		memo_allarmi_5; //stato degli allarmi, bit per bit
	tstamp_t tstamp_ack; //timestamp utimo ack
	u_short memo_ack_1,
		memo_ack_2,
		memo_ack_3,
		memo_ack_4,
		memo_ack_5; //stato degli acknowledge, bit per bit
	u_short mesg_type; //type of the message: hearthbeat or change
	u_long seq_num,		//sequence number
		seq_async,	//sequence number change mesg
		seq_sent;	//sequence number sent packets
	u_short db_number, //number of db(check)   
		spare;
} intlck_libera_t;

typedef struct ilk_answer {
	u_short code;
	u_short ack_1,
		ack_2,
		ack_3,
		ack_4,
		ack_5; 
	tstamp_t tstamp_srv; 
} ilk_answer_t;

static int maskbits[] = {
	MASK_BIT0, 
	MASK_BIT1, 
	MASK_BIT2, 
	MASK_BIT3, 
	MASK_BIT4, 
	MASK_BIT5, 
	MASK_BIT6, 
	MASK_BIT7, 
	MASK_BIT8, 
	MASK_BIT9, 
	MASK_BIT10, 
	MASK_BIT11, 
	MASK_BIT12, 
	MASK_BIT13, 
	MASK_BIT14, 
	MASK_BIT15
};

static db_mesg openid_m[][2] = {
	{"ID S1.1 CLOSED","ID S1.1 OPENED"},
	{"ID S1.2 CLOSED","ID S1.2 OPENED"},
	{"ID S2.2 CLOSED","ID S2.2 OPENED"},
	{"ID S3.2 CLOSED","ID S3.2 OPENED"},
	{"ID S4.2 CLOSED","ID S4.2 OPENED"},
	{"ID S5.2 CLOSED","ID S5.2 OPENED"},
	{"ID S6.2 CLOSED","ID S6.2 OPENED"},
	{"ID S7.2 CLOSED","ID S7.2 OPENED"},
	{"ID S8.2 CLOSED","ID S8.2 OPENED"},
	{"ID S9.2 CLOSED","ID S9.2 OPENED"},
	{"ID S10.2 CLOSED","ID S10.2 OPENED"},
	{"ID S11.2 CLOSED","ID S11.2 OPENED"},
	{" "," "},
	{" "," "},
	{" "," "},
	{" "," "}
};

static db_mesg diodedet_m[][2] = {
	{"DIODE BPM S1.1 ALARM","DIODE BPM S1.1 NORMAL"},
	{"DIODE BPM S1.2 ALARM","DIODE BPM S1.2 NORMAL"},
	{"DIODE BPM S2.2 ALARM","DIODE BPM S2.2 NORMAL"},
	{"DIODE BPM S3.2 ALARM","DIODE BPM S3.2 NORMAL"},
	{"DIODE BPM S4.2 ALARM","DIODE BPM S4.2 NORMAL"},
	{"DIODE BPM S5.2 ALARM","DIODE BPM S5.2 NORMAL"},
	{"DIODE BPM S6.2 ALARM","DIODE BPM S6.2 NORMAL"},
	{"DIODE BPM S7.2 ALARM","DIODE BPM S7.2 NORMAL"},
	{"DIODE BPM S8.2 ALARM","DIODE BPM S8.2 NORMAL"},
	{"DIODE BPM S9.2 ALARM","DIODE BPM S9.2 NORMAL"},
	{"DIODE BPM S10.2 ALARM","DIODE BPM S10.2 NORMAL"},
	{"DIODE BPM S11.2 ALARM","DIODE BPM S11.2 NORMAL"},
	{" "," "},
	{" "," "},
	{" "," "},
	{" "," "}
};

static db_mesg liberast_m[][2] = {
	{"Libera BPM S12.8 ALARM","Libera BPM S12.8 NORMAL"},
	{"Libera BPM S1.3 ALARM","Libera BPM S1.3 NORMAL"},
	{"Libera BPM S2.3 ALARM","Libera BPM S2.3 NORMAL"},
	{"Libera BPM S3.3 ALARM","Libera BPM S3.3 NORMAL"},
	{"Libera BPM S4.3 ALARM","Libera BPM S4.3 NORMAL"},
	{"Libera BPM S5.3 ALARM","Libera BPM S5.3 NORMAL"},
	{"Libera BPM S6.3 ALARM","Libera BPM S6.3 NORMAL"},
	{"Libera BPM S7.3 ALARM","Libera BPM S7.3 NORMAL"},
	{"Libera BPM S8.3 ALARM","Libera BPM S8.3 NORMAL"},
	{"Libera BPM S9.3 ALARM","Libera BPM S9.3 NORMAL"},
	{"Libera BPM S10.3 ALARM","Libera BPM S10.3 NORMAL"},
	{"Libera BPM S11.3 ALARM","Libera BPM S11.3 NORMAL"},
	{" "," "},
	{" "," "},
	{" "," "},
	{" "," "}
};

static db_mesg liberand_m[][2] = {
	{"Libera BPM S1.1 ALARM","Libera BPM S1.1 NORMAL"},
	{"Libera BPM S1.4 ALARM","Libera BPM S1.4 NORMAL"},
	{"Libera BPM S2.4 ALARM","Libera BPM S2.4 NORMAL"},
	{"Libera BPM S3.4 ALARM","Libera BPM S3.4 NORMAL"},
	{"Libera BPM S4.4 ALARM","Libera BPM S4.4 NORMAL"},
	{"Libera BPM S5.4 ALARM","Libera BPM S5.4 NORMAL"},
	{"Libera BPM S6.4 ALARM","Libera BPM S6.4 NORMAL"},
	{"Libera BPM S7.4 ALARM","Libera BPM S7.4 NORMAL"},
	{"Libera BPM S8.4 ALARM","Libera BPM S8.4 NORMAL"},
	{"Libera BPM S9.4 ALARM","Libera BPM S9.4 NORMAL"},
	{"Libera BPM S10.4 ALARM","Libera BPM S10.4 NORMAL"},
	{"Libera BPM S11.4 ALARM","Libera BPM S11.4 NORMAL"},
	{" "," "},
	{" "," "},
	{" "," "},
	{" "," "}
};

static db_mesg ithreshold_m[] = {
	"I > 20 mA",
	"I < 20 mA"
};
static db_mesg libera3hc_m[] = {
	"Libera BPM S11.2 (3HC) ALARM",
	"Libera BPM S11.2 (3HC) NORMAL"
};
static db_mesg plca10_m[] = {
	"beam dump from A10 PLC",
	"beam enabled from A10 PLC"
};
static db_mesg beam_dump_cmd = "beam dump command";
static db_mesg beam_dump_m[] = {
	"beam dump",
	"beam enabled"
};

typedef struct dbm{
	db_mesg mesg_openid[NUM_ID][2],
		mesg_diodedet[NUM_DIODE][2],
		mesg_liberast[NUM_LIBERA1][2],
		mesg_liberand[NUM_LIBERA2][2],
		mesg_ithreshold[2],
		mesg_libera3hc[2],
		mesg_plca10[2],
		mesg_beamabi[2];
} dbm_;

typedef struct att_db {
	char *att_name;//name of the attribute
	db_mesg *fixed_m;//fixed messages for the specific attribute
	db_mesg *res_m;//resulting messages
	int dim;
} att_db_;

/*----- PROTECTED REGION END -----*/	//	PLCsocket::Additional Class Declarations

class PLCsocket : public TANGO_BASE_CLASS
{

/*----- PROTECTED REGION ID(PLCsocket::Data Members) ENABLED START -----*/

//	Add your own data members
public:



/*----- PROTECTED REGION END -----*/	//	PLCsocket::Data Members

//	Device property data members
public:
	//	Ipaddr:	ip address of PLC
	string	ipaddr;
	//	Wport:	
	Tango::DevShort	wport;
	//	Rport:	
	Tango::DevShort	rport;
	//	Srv_host:	ip addr of the server
	string	srv_host;
	//	Hbeat:	max time in ms between two read from plc
	Tango::DevShort	hbeat;
	//	Labels:	list of labels to be written in the panel
	vector<string>	labels;
	//	Logfile:	
	string	logfile;
	//	Db_param:	contains the parameter string to access logging db:<BR>
	//  hostname-user-passwd-db_name-port
	vector<string>	db_param;
	//	Logging:	
	Tango::DevShort	logging;
	//	Timeadjust:	
	string	timeadjust;
	//	Ack_policy:	
	string	ack_policy;

//	Attribute data members
public:
	Tango::DevBoolean	*attr_IThreshold_read;
	Tango::DevBoolean	*attr_Libera3hc_read;
	Tango::DevBoolean	*attr_BeamDumpPLCA10_read;
	Tango::DevBoolean	*attr_BeamAbi_read;
	Tango::DevBoolean	*attr_CallPm_read;
	Tango::DevBoolean	*attr_AlIThreshold_read;
	Tango::DevBoolean	*attr_AlLibera3hc_read;
	Tango::DevString	*attr_Seq_state_read;
	Tango::DevBoolean	*attr_EnableBPM11_2_read;
	Tango::DevBoolean	*attr_OpenId_read;
	Tango::DevBoolean	*attr_DiodeDet_read;
	Tango::DevBoolean	*attr_LiberaSt_read;
	Tango::DevBoolean	*attr_LiberaNd_read;
	Tango::DevBoolean	*attr_IlkEnable_read;
	Tango::DevBoolean	*attr_AlOpenId_read;
	Tango::DevBoolean	*attr_AlDiodeDet_read;
	Tango::DevBoolean	*attr_AlLiberaSt_read;
	Tango::DevBoolean	*attr_AlLiberaNd_read;
	Tango::DevDouble	*attr_Seq_num_read;

//	Constructors and destructors
public:
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	PLCsocket(Tango::DeviceClass *cl,string &s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	PLCsocket(Tango::DeviceClass *cl,const char *s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device name
	 *	@param d	Device description.
	 */
	PLCsocket(Tango::DeviceClass *cl,const char *s,const char *d);
	/**
	 * The device object destructor.
	 */
	~PLCsocket() {delete_device();};


//	Miscellaneous methods
public:
	/*
	 *	will be called at device destruction or at init command.
	 */
	void delete_device();
	/*
	 *	Initialize the device
	 */
	virtual void init_device();
	/*
	 *	Read the device properties from database
	 */
	void get_device_property();
	/*
	 *	Always executed method before execution command method.
	 */
	virtual void always_executed_hook();


//	Attribute methods
public:
	//--------------------------------------------------------
	/*
	 *	Method      : PLCsocket::read_attr_hardware()
	 *	Description : Hardware acquisition for attributes.
	 */
	//--------------------------------------------------------
	virtual void read_attr_hardware(vector<long> &attr_list);

/**
 *	Attribute IThreshold related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_IThreshold(Tango::Attribute &attr);
	virtual bool is_IThreshold_allowed(Tango::AttReqType type);
/**
 *	Attribute Libera3hc related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_Libera3hc(Tango::Attribute &attr);
	virtual bool is_Libera3hc_allowed(Tango::AttReqType type);
/**
 *	Attribute BeamDumpPLCA10 related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_BeamDumpPLCA10(Tango::Attribute &attr);
	virtual bool is_BeamDumpPLCA10_allowed(Tango::AttReqType type);
/**
 *	Attribute BeamAbi related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_BeamAbi(Tango::Attribute &attr);
	virtual bool is_BeamAbi_allowed(Tango::AttReqType type);
/**
 *	Attribute CallPm related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_CallPm(Tango::Attribute &attr);
	virtual bool is_CallPm_allowed(Tango::AttReqType type);
/**
 *	Attribute AlIThreshold related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_AlIThreshold(Tango::Attribute &attr);
	virtual bool is_AlIThreshold_allowed(Tango::AttReqType type);
/**
 *	Attribute AlLibera3hc related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_AlLibera3hc(Tango::Attribute &attr);
	virtual bool is_AlLibera3hc_allowed(Tango::AttReqType type);
/**
 *	Attribute Seq_state related methods
 *	Description: 
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
	virtual void read_Seq_state(Tango::Attribute &attr);
	virtual bool is_Seq_state_allowed(Tango::AttReqType type);
/**
 *	Attribute EnableBPM11_2 related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_EnableBPM11_2(Tango::Attribute &attr);
	virtual bool is_EnableBPM11_2_allowed(Tango::AttReqType type);
/**
 *	Attribute OpenId related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
	virtual void read_OpenId(Tango::Attribute &attr);
	virtual bool is_OpenId_allowed(Tango::AttReqType type);
/**
 *	Attribute DiodeDet related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
	virtual void read_DiodeDet(Tango::Attribute &attr);
	virtual bool is_DiodeDet_allowed(Tango::AttReqType type);
/**
 *	Attribute LiberaSt related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
	virtual void read_LiberaSt(Tango::Attribute &attr);
	virtual bool is_LiberaSt_allowed(Tango::AttReqType type);
/**
 *	Attribute LiberaNd related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
	virtual void read_LiberaNd(Tango::Attribute &attr);
	virtual bool is_LiberaNd_allowed(Tango::AttReqType type);
/**
 *	Attribute IlkEnable related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
	virtual void read_IlkEnable(Tango::Attribute &attr);
	virtual bool is_IlkEnable_allowed(Tango::AttReqType type);
/**
 *	Attribute AlOpenId related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
	virtual void read_AlOpenId(Tango::Attribute &attr);
	virtual bool is_AlOpenId_allowed(Tango::AttReqType type);
/**
 *	Attribute AlDiodeDet related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
	virtual void read_AlDiodeDet(Tango::Attribute &attr);
	virtual bool is_AlDiodeDet_allowed(Tango::AttReqType type);
/**
 *	Attribute AlLiberaSt related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
	virtual void read_AlLiberaSt(Tango::Attribute &attr);
	virtual bool is_AlLiberaSt_allowed(Tango::AttReqType type);
/**
 *	Attribute AlLiberaNd related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 16
 */
	virtual void read_AlLiberaNd(Tango::Attribute &attr);
	virtual bool is_AlLiberaNd_allowed(Tango::AttReqType type);
/**
 *	Attribute Seq_num related methods
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 3
 */
	virtual void read_Seq_num(Tango::Attribute &attr);
	virtual bool is_Seq_num_allowed(Tango::AttReqType type);


	//--------------------------------------------------------
	/**
	 *	Method      : PLCsocket::add_dynamic_attributes()
	 *	Description : Add dynamic attributes if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_attributes();




//	Command related methods
public:
	/**
	 *	Command Beam_dump related method
	 *	Description: 
	 *
	 *	@returns 
	 */
	virtual Tango::DevString beam_dump();
	virtual bool is_Beam_dump_allowed(const CORBA::Any &any);
	/**
	 *	Command Send_ack related method
	 *	Description: 
	 *
	 *	@returns 
	 */
	virtual Tango::DevString send_ack();
	virtual bool is_Send_ack_allowed(const CORBA::Any &any);
	/**
	 *	Command Seq_state_rst related method
	 *	Description: 
	 *
	 */
	virtual void seq_state_rst();
	virtual bool is_Seq_state_rst_allowed(const CORBA::Any &any);
	/**
	 *	Command Test_debug related method
	 *	Description: 
	 *
	 */
	virtual void test_debug();
	virtual bool is_Test_debug_allowed(const CORBA::Any &any);
	/**
	 *	Command EnableBPM11_2 related method
	 *	Description: 
	 *
	 *	@param argin 
	 */
	virtual void enable_bpm11_2(Tango::DevBoolean argin);
	virtual bool is_EnableBPM11_2_allowed(const CORBA::Any &any);
	/**
	 *	Command DisableInterlockSCW related method
	 *	Description: 
	 *
	 */
	virtual void disable_interlock_scw();
	virtual bool is_DisableInterlockSCW_allowed(const CORBA::Any &any);
	/**
	 *	Command DisableInterlockID5 related method
	 *	Description: 
	 *
	 */
	virtual void disable_interlock_id5();
	virtual bool is_DisableInterlockID5_allowed(const CORBA::Any &any);


	//--------------------------------------------------------
	/**
	 *	Method      : PLCsocket::add_dynamic_commands()
	 *	Description : Add dynamic commands if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_commands();

/*----- PROTECTED REGION ID(PLCsocket::Additional Method prototypes) ENABLED START -----*/

//	Additional Method prototypes
int abortflag;  // flag used to abort thread loops 	
		
	plclgthread *logloop; //logging thread
	plctimingthread *tmloop; //timing thread
	plcsrvthread *srvloop;	//read from socket thread
	omni_mutex::omni_mutex *mutex;
	deque<intlck_libera_t> mesg_list;
	
	union semun {
		int val;
		struct semid_ds *buf;
		ushort * array;
	} argument;
	int semid;


	string ip_host;		// ip host name for tcp/ip 
	int new_sd, sock;	// socket for data exchange
	int fdlog;		//logfile
	const char* logfile2;	//name of the logfile

	MYSQL dbp;

	void log_it();//logging of the raw message
	void log_msg(intlck_libera_t m);//logging of the raw message
	void log_to_db(db_mesg ms, tstamp_t t);
	void push_it();//push of the events to subscribed clients
	void sync_plc(tstamp_t t);
	
	int is_beamdumpcmd;
	int logging_msg;
	intlck_libera_t message;
	intlck_libera_t old_message;
	tstamp_t old_timestamp;
	tstamp_t new_timestamp;
	tstamp_t last_ck_read;
	ilk_answer_t i_answer;
	
	//data from field
	Tango::DevBoolean openid[NUM_ID];
	Tango::DevBoolean diodedet[NUM_DIODE];
	Tango::DevBoolean liberast[NUM_LIBERA1];
	Tango::DevBoolean liberand[NUM_LIBERA2];
	Tango::DevBoolean IlkEnable[NUM_INTLOCK];
	Tango::DevBoolean ithreshold;
	Tango::DevBoolean libera3hc;
	Tango::DevBoolean beamdumpplca10;
	Tango::DevBoolean beamabi;
	Tango::DevBoolean callpm;
	Tango::DevBoolean corr_liberast[NUM_LIBERA1];
	Tango::DevBoolean corr_liberand[NUM_LIBERA2];
	Tango::DevBoolean corr_libera3hc;
	Tango::DevDouble seq[3];
	Tango::DevString s_state;
	
	//alarms
	Tango::DevBoolean al_openid[NUM_ID];
	Tango::DevBoolean al_diodedet[NUM_DIODE];
	Tango::DevBoolean al_liberast[NUM_LIBERA1];
	Tango::DevBoolean al_liberand[NUM_LIBERA2];
	Tango::DevBoolean al_ithreshold;
	Tango::DevBoolean al_libera3hc;
	
	//acknowledges registered from plc
	Tango::DevBoolean ack_openid[NUM_ID];
	Tango::DevBoolean ack_diodedet[NUM_DIODE];
	Tango::DevBoolean ack_liberast[NUM_LIBERA1];
	Tango::DevBoolean ack_liberand[NUM_LIBERA2];
	Tango::DevBoolean ack_ithreshold;
	Tango::DevBoolean ack_libera3hc;
	Tango::DevBoolean ack_beamdumpplca10;

	//old message for compare
	Tango::DevBoolean old_openid[NUM_ID];
	Tango::DevBoolean old_diodedet[NUM_DIODE];
	Tango::DevBoolean old_liberast[NUM_LIBERA1];
	Tango::DevBoolean old_liberand[NUM_LIBERA2];
	Tango::DevBoolean old_IlkEnable[NUM_INTLOCK];
	Tango::DevBoolean old_ithreshold;
	Tango::DevBoolean old_libera3hc;
	Tango::DevBoolean old_beamdumpplca10;
	Tango::DevBoolean old_beamabi;
	Tango::DevBoolean old_corr_liberast[NUM_LIBERA1];
	Tango::DevBoolean old_corr_liberand[NUM_LIBERA2];
	Tango::DevBoolean old_corr_libera3hc;

	//log messages from db
	db_mesg mesg_openid[NUM_ID][2];
	db_mesg mesg_diodedet[NUM_DIODE][2];
	db_mesg mesg_liberast[NUM_LIBERA1][2];
	db_mesg mesg_liberand[NUM_LIBERA2][2];
	db_mesg mesg_ithreshold[2];
	db_mesg mesg_libera3hc[2];
	db_mesg mesg_plca10[2];
	db_mesg mesg_beamabi[2];
	dbm dbmessages;
	
	struct timeval _pre2, _post2;
	char buffer[BUF_LEN];
	uint16_t ack_mask[5];
	void send_auto_ack(void);
	string calc_timest();
	long int calc_timeval(tstamp_t);
	long int calc_time_ms(tstamp_t);


protected :	
	void send_frame (unsigned char *frame, short frame_length);

/*----- PROTECTED REGION END -----*/	//	PLCsocket::Additional Method prototypes
};

/*----- PROTECTED REGION ID(PLCsocket::Additional Classes Definitions) ENABLED START -----*/

//	Additional Classes Definitions

/*----- PROTECTED REGION END -----*/	//	PLCsocket::Additional Classes Definitions

}	//	End of namespace

#endif   //	PLCsocket_H
